@@ -10,6 +10,7 @@ import os
 import pathlib
 import random
 import shutil
+import struct
 import subprocess
 import sys
 import time
@@ -644,6 +645,93 @@ async def asyncExecute(*parList):
     return proc.returncode, stdout, stderr
 
 
+class BytesWriter():
+
+    def __init__(self, isBigEndian=True):
+        self.isBigEndian = isBigEndian
+        self.formatAry = []
+        self.valueAry = []
+
+    def toBytes(self):
+        formatStr = self.isBigEndian and ">" or "<"
+        formatStr += "".join(self.formatAry)
+        return struct.pack(formatStr, *self.valueAry)
+
+    def write(self, format, value):
+        self.formatAry.append(format)
+        self.valueAry.append(value)
+
+    def writeAry(self, func, ary):
+        self.writeInt32(len(ary))
+        for value in ary:
+            func(value)
+
+    # -----
+
+    def writeInt16(self, value):
+        self.write("h", value)  # -32768 ~ +32767
+
+    def writeUInt16(self, value):
+        self.write("H", value)  # 0 ~ 65535
+
+    def writeInt32(self, value):
+        self.write("i", value)  # -2147483648 ~ +2147483647
+
+    def writeUInt32(self, value):
+        self.write("I", value)  # 0 ~ 4294967295
+
+    def writeInt64(self, value):
+        self.write("q", value)  # -9223372036854775808 ~ +9223372036854775807
+
+    def writeUInt64(self, value):
+        self.write("Q", value)  # 0 ~ 18446744073709551615
+
+    def writeFloat(self, value):
+        self.write("f", value)
+
+    def writeDouble(self, value):
+        self.write("d", value)
+
+    def writeBool(self, value):
+        self.write("?", value)
+
+    def writeStr(self, value):
+        value = value.encode("utf8")
+        self.write(f"{len(value)+1}p", value)
+
+    # -----
+
+    def writeInt16Ary(self, ary):
+        self.writeAry(self.writeInt16, ary)
+
+    def writeUInt16Ary(self, ary):
+        self.writeAry(self.writeUInt16, ary)
+
+    def writeInt32Ary(self, ary):
+        self.writeAry(self.writeInt32, ary)
+
+    def writeUInt32Ary(self, ary):
+        self.writeAry(self.writeUInt32, ary)
+
+    def writeInt64Ary(self, ary):
+        self.writeAry(self.writeInt64, ary)
+
+    def writeUInt64Ary(self, ary):
+        self.writeAry(self.writeUInt64, ary)
+
+    def writeFloatAry(self, ary):
+        self.writeAry(self.writeFloat, ary)
+
+    def writeDoubleAry(self, ary):
+        self.writeAry(self.writeDouble, ary)
+
+    def writeBoolAry(self, ary):
+        self.writeAry(self.writeBool, ary)
+
+    def writeStrAry(self, ary):
+        self.writeAry(self.writeStr, ary)
+
+
 _maxAsyncHttpNum = 3
 _currentAsyncHttpNum = 0
 _waitAsyncHttpTime = 0.1
